#include <stdlib.h>
#include "main.h"
#include "uart.h"


#define KEYPORT PORTB
#define KEYDDR  DDRB
#define KEYPIN  PINB
#define KEY0    PB0		// capture input


#define LEDPORT	PORTC
#define LEDDDR	DDRC
#define	LED0	PC0


volatile u16	capval;


void t1_init( void )
{
  TCCR1B = 1<<ICNC1^1<<ICES1^1<<CS10;		// noise canceler
						// rising edge
						// prescaler 1:1
  TIMSK = 1<<TOIE1;
}


SIGNAL( SIG_OVERFLOW1 )
{
  static u16 chargestart;

  if( KEYDDR & 1<<KEY0 ){			// discharge done
    TIFR = 1<<ICF1;				// clear cature flag
    KEYDDR &= ~(1<<KEY0);			// discharge off
    chargestart = TCNT1;			// charging start value
    KEYPORT |= 1<<KEY0;				// pull up on
  }else{
    if( TIFR & 1<<ICF1 )			// charging done
      capval = ICR1 - chargestart;		// get charging time
    KEYPORT &= ~(1<<KEY0);			// prepare discharge
    KEYDDR |= 1<<KEY0;				// discharge
  }
}


int main( void )
{
  u16 i;
  u8 s[10];

  t1_init();
  uinit();
  sei();
  uputs("Test Sensor Key:\n\r");

  for(;;){
    if( capval ){
      cli();
      i = capval;
      capval = 0;
      sei();
      if( i < 25 )				// STK500 at 11MHz !
        LEDDDR &= ~(1<<LED0);
      else
        LEDDDR |= 1<<LED0;
      itoa( i, s, 10 );
      uputs( s );
      uputs( "\n\r" );
    }
  }
}
